//*****************************************************************************
//
//! @file ble_freertos_fit_lp.c
//!
//! @brief ARM Cordio BLE - Fit Application Example.
//!
//! Purpose: This example implements a BLE heart rate sensor within the FreeRTOS
//! framework. To minimize power usage, this application is compiled without
//! debug printing by default (the "lp" version of this example excludes
//! them by default).
//!
//! Additional Information:
//! To enable debug printing, add the following project-level macro definitions.
//!
//! AM_DEBUG_PRINTF
//! WSF_TRACE_ENABLED=1
//!
//! When defined, debug messages will be sent over ITM/SWO at 1M Baud.
//!
//! Note that when these macros are defined, the device will never achieve deep
//! sleep, only normal sleep, due to the ITM (and thus the HFRC) being enabled.
//
//*****************************************************************************

//*****************************************************************************
//
// Copyright (c) 2020, Ambiq Micro
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// 3. Neither the name of the copyright holder nor the names of its
// contributors may be used to endorse or promote products derived from this
// software without specific prior written permission.
//
// Third party software included in this distribution is subject to the
// additional license terms as defined in the /docs/licenses directory.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
// This is part of revision v2.4.2-1403-gd656d944e of the AmbiqSuite Development Package.
//
//*****************************************************************************

//*****************************************************************************
//
// This application has a large number of common include files. For
// convenience, we'll collect them all together in a single header and include
// that everywhere.
//
//*****************************************************************************
#include "ble_freertos_fit_lp.h"
#include "rtos.h"

//*****************************************************************************
//
// Enable printing to the console.
//
//*****************************************************************************
void
enable_print_interface(void)
{
    //
    // Initialize a debug printing interface.
    //
    am_bsp_itm_printf_enable();
}

//*****************************************************************************
//
// Main Function
//
//*****************************************************************************
int
main(void)
{
  am_hal_pwrctrl_mcu_memory_config_t McuMemCfg = 
  {
    .eCacheCfg    = AM_HAL_PWRCTRL_CACHE_ALL,
    .bRetainCache = true,
    .eDTCMCfg     = AM_HAL_PWRCTRL_DTCM_128K,
    .eRetainDTCM  = AM_HAL_PWRCTRL_DTCM_128K,
    .bEnableNVM0  = true,
    .bRetainNVM0  = true
  };
  
  am_hal_pwrctrl_sram_memcfg_t SRAMMemCfg =
  {
    .eSRAMCfg         = AM_HAL_PWRCTRL_SRAM_NONE,
    .eActiveWithMCU   = AM_HAL_PWRCTRL_SRAM_NONE,
    .eActiveWithDSP   = AM_HAL_PWRCTRL_SRAM_NONE,
    .eSRAMRetain      = AM_HAL_PWRCTRL_SRAM_NONE
  };
  
#ifndef NOFPU
  //
  // Enable the floating point module, and configure the core for lazy
  // stacking.
  //
  am_hal_sysctrl_fpu_enable();
  am_hal_sysctrl_fpu_stacking_enable(true);
#else
  am_hal_sysctrl_fpu_disable();
#endif
  
  //
  // Configure the board for low power.
  //
  am_bsp_low_power_init();
  
  //
  // Turn off unneeded memory
  //
  am_hal_pwrctrl_mcu_memory_config(&McuMemCfg);
  am_hal_pwrctrl_sram_config(&SRAMMemCfg);
  
  // Enable printing to the console.
  //
#ifdef AM_DEBUG_PRINTF
  enable_print_interface();
  //
  // Print the banner.
  //
  am_util_stdio_terminal_clear();
  am_util_debug_printf("FreeRTOS Fit Low Power Example\n");
  
#endif
  
  //
  // Run the application.
  //
  run_tasks();
  
  //
  // We shouldn't ever get here.
  //
  while (1)
  {
  }
  
}

